import React, { useCallback, useState, useMemo } from 'react';
import { scaleTime, extent, scaleLog, max, line } from 'd3';
import { XAxis } from './XAxis';
import { YAxis } from './YAxis';
import { VoronoiOverlay } from './VoronoiOverlay';

const xValue = (d) => d.date;
const yValue = (d) => d.deathTotal;

const margin = { top: 40, left: 80, bottom: 50, right: 80 };

export const LineChart = ({ data, height, width }) => {
  const [
    activeCountryName,
    setActiveCountryName,
  ] = useState();

  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom;

  const allData = useMemo(
    () => 
    	data.reduce((accumulator, countryTimeseries) =>
      	accumulator.concat(countryTimeseries),
    	[]	),
    [data]);  
  const epsilon = 1;
  const xScale = useMemo(
    () => 
    	scaleTime()
    		.domain(extent(allData, xValue))
    		.range([0, innerWidth]),
    [allData, xValue, innerWidth]);

  const yScale = useMemo(
    () => 
    	scaleLog()
    		.domain([epsilon, max(allData, yValue)])
    		.range([innerHeight, 0])
    		.nice(),
    [allData, yValue, innerHeight]);

  const lineGenerator = useMemo(
    () =>
    	line()
    		.x((d) => xScale(xValue(d)))
    		.y((d) => yScale(epsilon + yValue(d))),
    [xScale, yScale, xValue, yValue, epsilon]);

  const mostRecentDate = xScale.domain()[1];

  console.log(activeCountryName);

  const handelVoronoiHover = useCallback((d) => {
    console.log('Hovered');
    console.log(d);
    setActiveCountryName(d.countryName);
  }, []);

  return (
    <svg width={width} height={height}>
      <g
        transform={`translate(${margin.left}, ${margin.top})`}
      >
        <XAxis xScale={xScale} innerHeight={innerHeight} />
        <YAxis yScale={yScale} innerWidth={innerWidth} />
        {data.map((countryTimeseries) => {
          return (
            <path
              className="marker-line"
              d={lineGenerator(countryTimeseries)}
            />
          );
        })}
        {activeCountryName && (
          <path
            className="marker-line active"
            d={lineGenerator(
              data.find(
                (countryTimeSeries) =>
                  countryTimeSeries.countryName === activeCountryName
              )
            )}
          />
        )}
        <text
          transform={`translate(${innerWidth / 2}, -10)`}
          text-anchor="middle"
        >
          Global Coronavirus Deaths Over Time By Country
        </text>
        <text
          className="axis-label"
          transform={`translate(-40, ${
            innerHeight / 2
          }) rotate(-90)`}
          text-anchor="middle"
        >
          Cumulative Deaths
        </text>
        <text
          className="axis-label"
          transform={`translate(${innerWidth / 2}, ${
            innerHeight + 25
          })`}
          text-anchor="middle"
          alignment-baseline="hanging"
        >
          Time
        </text>
        <VoronoiOverlay
          onHover={handelVoronoiHover}
          innerWidth={innerWidth}
          innerHeight={innerHeight}
          allData={allData}
          lineGenerator={lineGenerator}
        />
      </g>
    </svg>
  );
};
